@@ -20,14 +20,17 @@
 
 -- imports
 import("core.base.option")
+import("core.base.semver")
 import("core.tool.toolchain")
 import("core.project.config")
 import("core.tool.linker")
 import("core.tool.compiler")
+import("core.project.project")
 import("lib.detect.find_file")
 import("lib.detect.find_tool")
 import("package.tools.ninja")
 import("detect.sdks.find_emsdk")
+import("private.utils.toolchain", {alias = "toolchain_utils"})
 
 -- get the number of parallel jobs
 function _get_parallel_njobs(opt)
@@ -73,29 +76,10 @@ function _map_linkflags(package, targetkind, sourcekinds, name, values)
     return linker.map_flags(targetkind, sourcekinds, name, values, {target = package})
 end
 
--- is cross compilation?
-function _is_cross_compilation(package)
-    if not package:is_plat(os.subhost()) then
-        return true
-    end
-    if package:is_plat("macosx") and not package:is_arch(os.subarch()) then
-        return true
-    end
-    return false
-end
-
 -- is the toolchain compatible with the host?
 function _is_toolchain_compatible_with_host(package)
-    local toolchains = package:config("toolchains")
-    if toolchains then
-        toolchains = table.wrap(toolchains)
-        if is_host("linux", "macosx", "bsd") then
-            for _, name in ipairs(toolchains) do
-                if name:startswith("clang") or name:startswith("gcc") then
-                    return true
-                end
-            end
-        elseif is_host("windows") and table.contains(toolchains, "msvc") then
+    for _, name in ipairs(package:config("toolchains")) do
+        if toolchain_utils.is_compatible_with_host(name) then
             return true
         end
     end
@@ -103,8 +87,8 @@ end
 
 -- get msvc
 function _get_msvc(package)
-    local msvc = toolchain.load("msvc", {plat = package:plat(), arch = package:arch()})
-    assert(msvc:check(), "vs not found!") -- we need check vs envs if it has been not checked yet
+    local msvc = package:toolchain("msvc") or toolchain.load("msvc", {plat = package:plat(), arch = package:arch()})
+    assert(msvc:check(), "vs not found!") -- we need to check vs envs if it has been not checked yet
     return msvc
 end
 
@@ -180,6 +164,9 @@ function _get_cflags(package, opt)
     if package:config("lto") then
         table.join2(result, package:_generate_lto_configs("cc").cflags)
     end
+    if package:config("asan") then
+        table.join2(result, package:_generate_sanitizer_configs("address", "cc").cflags)
+    end
     table.join2(result, _get_cflags_from_packagedeps(package, opt))
     if #result > 0 then
         return os.args(result)
@@ -208,6 +195,9 @@ function _get_cxxflags(package, opt)
     if package:config("lto") then
         table.join2(result, package:_generate_lto_configs("cxx").cxxflags)
     end
+    if package:config("asan") then
+        table.join2(result, package:_generate_sanitizer_configs("address", "cxx").cxxflags)
+    end
     table.join2(result, _get_cflags_from_packagedeps(package, opt))
     if #result > 0 then
         return os.args(result)
@@ -243,9 +233,13 @@ function _get_ldflags(package, opt)
         table.join2(result, _map_linkflags(package, "binary", {"cxx"}, "syslink", package:build_getenv("syslinks")))
         table.join2(result, _map_linkflags(package, "binary", {"cxx"}, "linkdir", package:build_getenv("linkdirs")))
     end
+    table.join2(result, package:config("ldflags"))
     if package:config("lto") then
         table.join2(result, package:_generate_lto_configs().ldflags)
     end
+    if package:config("asan") then
+        table.join2(result, package:_generate_sanitizer_configs("address").ldflags)
+    end
     table.join2(result, _get_ldflags_from_packagedeps(package, opt))
     if opt.ldflags then
         table.join2(result, opt.ldflags)
@@ -265,9 +259,13 @@ function _get_shflags(package, opt)
         table.join2(result, _map_linkflags(package, "shared", {"cxx"}, "syslink", package:build_getenv("syslinks")))
         table.join2(result, _map_linkflags(package, "shared", {"cxx"}, "linkdir", package:build_getenv("linkdirs")))
     end
+    table.join2(result, package:config("shflags"))
     if package:config("lto") then
         table.join2(result, package:_generate_lto_configs().shflags)
     end
+    if package:config("asan") then
+        table.join2(result, package:_generate_sanitizer_configs("address").shflags)
+    end
     table.join2(result, _get_ldflags_from_packagedeps(package, opt))
     if opt.shflags then
         table.join2(result, opt.shflags)
@@ -277,6 +275,19 @@ function _get_shflags(package, opt)
     end
 end
 
+-- get cmake version
+function _get_cmake_version()
+    local cmake_version = _g.cmake_version
+    if not cmake_version then
+        local cmake = find_tool("cmake", {version = true})
+        if cmake and cmake.version then
+            cmake_version = semver.new(cmake.version)
+        end
+        _g.cmake_version = cmake_version
+    end
+    return cmake_version
+end
+
 -- get vs toolset
 function _get_vs_toolset(package)
     local toolset_ver = nil
@@ -287,9 +298,30 @@ function _get_vs_toolset(package)
             toolset_ver = "v" .. verinfo[1] .. (verinfo[2]:sub(1, 1) or "0")
         end
     end
+    -- cmake does not support vs toolset v144 below 3.29.0, we can only use v143
+    -- @see https://github.com/xmake-io/xmake/issues/4772
+    if toolset_ver and toolset_ver >= "v144" then
+        local cmake_version = _get_cmake_version()
+        if cmake_version and cmake_version:le("3.29.0") then
+            toolset_ver = "v143"
+        end
+    end
     return toolset_ver
 end
 
+-- insert configs from envs
+function _insert_configs_from_envs(configs, envs, opt)
+    opt = opt or {}
+    local configs_str = opt._configs_str
+    for k, v in pairs(envs) do
+        if configs_str and configs_str:find(k, 1, true) then
+            -- use user custom configuration
+        else
+            table.insert(configs, "-D" .. k .. "=" .. v)
+        end
+    end
+end
+
 -- get configs for generic
 function _get_configs_for_generic(package, configs, opt)
     local cflags = _get_cflags(package, opt)
@@ -342,24 +374,28 @@ function _get_configs_for_windows(package, configs, opt)
     -- we maybe need patch `cmake_policy(SET CMP0091 NEW)` to enable this argument for some packages
     -- @see https://cmake.org/cmake/help/latest/policy/CMP0091.html#policy:CMP0091
     -- https://github.com/xmake-io/xmake-repo/pull/303
-    local vs_runtime = package:config("vs_runtime")
-    if vs_runtime == "MT" then
+    local runtime
+    if package:has_runtime("MT") then
         table.insert(configs, "-DCMAKE_MSVC_RUNTIME_LIBRARY=MultiThreaded")
-    elseif vs_runtime == "MTd" then
+        runtime = "MT"
+    elseif package:has_runtime("MTd") then
         table.insert(configs, "-DCMAKE_MSVC_RUNTIME_LIBRARY=MultiThreadedDebug")
-    elseif vs_runtime == "MD" then
+        runtime = "MTd"
+    elseif package:has_runtime("MD") then
         table.insert(configs, "-DCMAKE_MSVC_RUNTIME_LIBRARY=MultiThreadedDLL")
-    elseif vs_runtime == "MDd" then
+        runtime = "MD"
+    elseif package:has_runtime("MDd") then
         table.insert(configs, "-DCMAKE_MSVC_RUNTIME_LIBRARY=MultiThreadedDebugDLL")
+        runtime = "MDd"
     end
-    if vs_runtime then
+    if runtime then
         -- CMake default MSVC flags as of 3.21.2
         local default_debug_flags = "/Zi /Ob0 /Od /RTC1"
         local default_release_flags = "/O2 /Ob2 /DNDEBUG"
-        table.insert(configs, '-DCMAKE_CXX_FLAGS_DEBUG=/' .. vs_runtime .. ' ' .. default_debug_flags)
-        table.insert(configs, '-DCMAKE_CXX_FLAGS_RELEASE=/' .. vs_runtime .. ' ' .. default_release_flags)
-        table.insert(configs, '-DCMAKE_C_FLAGS_DEBUG=/' .. vs_runtime .. ' ' .. default_debug_flags)
-        table.insert(configs, '-DCMAKE_C_FLAGS_RELEASE=/' .. vs_runtime .. ' ' .. default_release_flags)
+        table.insert(configs, '-DCMAKE_CXX_FLAGS_DEBUG=/' .. runtime .. ' ' .. default_debug_flags)
+        table.insert(configs, '-DCMAKE_CXX_FLAGS_RELEASE=/' .. runtime .. ' ' .. default_release_flags)
+        table.insert(configs, '-DCMAKE_C_FLAGS_DEBUG=/' .. runtime .. ' ' .. default_debug_flags)
+        table.insert(configs, '-DCMAKE_C_FLAGS_RELEASE=/' .. runtime .. ' ' .. default_release_flags)
     end
     if not opt._configs_str:find("CMAKE_COMPILE_PDB_OUTPUT_DIRECTORY") then
         table.insert(configs, "-DCMAKE_COMPILE_PDB_OUTPUT_DIRECTORY=pdb")
@@ -368,9 +404,9 @@ function _get_configs_for_windows(package, configs, opt)
 end
 
 -- get configs for android
+-- https://developer.android.google.cn/ndk/guides/cmake
 function _get_configs_for_android(package, configs, opt)
-
-    -- https://developer.android.google.cn/ndk/guides/cmake
+    opt = opt or {}
     local ndk = get_config("ndk")
     if ndk and os.isdir(ndk) then
         local ndk_sdkver = get_config("ndk_sdkver")
@@ -384,7 +420,7 @@ function _get_configs_for_android(package, configs, opt)
         if ndk_cxxstl then
             table.insert(configs, "-DANDROID_STL=" .. ndk_cxxstl)
         end
-        if is_host("windows") then
+        if is_host("windows") and opt.cmake_generator ~= "Ninja" then
             local make = path.join(ndk, "prebuilt", "windows-x86_64", "bin", "make.exe")
             if os.isfile(make) then
                 table.insert(configs, "-DCMAKE_MAKE_PROGRAM=" .. make)
@@ -419,15 +455,14 @@ function _get_configs_for_appleos(package, configs, opt)
     elseif package:is_plat("macosx") then
         envs.CMAKE_SYSTEM_NAME = "Darwin"
     end
+    envs.CMAKE_OSX_ARCHITECTURES = package:arch()
     envs.CMAKE_FIND_ROOT_PATH_MODE_LIBRARY   = "BOTH"
     envs.CMAKE_FIND_ROOT_PATH_MODE_INCLUDE   = "BOTH"
     envs.CMAKE_FIND_ROOT_PATH_MODE_FRAMEWORK = "BOTH"
     envs.CMAKE_FIND_ROOT_PATH_MODE_PROGRAM   = "NEVER"
     -- avoid install bundle targets
     envs.CMAKE_MACOSX_BUNDLE       = "NO"
-    for k, v in pairs(envs) do
-        table.insert(configs, "-D" .. k .. "=" .. v)
-    end
+    _insert_configs_from_envs(configs, envs, opt)
 end
 
 -- get configs for mingw
@@ -440,7 +475,6 @@ function _get_configs_for_mingw(package, configs, opt)
     envs.CMAKE_CXX_COMPILER        = _translate_bin_path(package:build_getenv("cxx"))
     envs.CMAKE_ASM_COMPILER        = _translate_bin_path(package:build_getenv("as"))
     envs.CMAKE_AR                  = _translate_bin_path(package:build_getenv("ar"))
-    envs.CMAKE_LINKER              = _translate_bin_path(package:build_getenv("ld"))
     envs.CMAKE_RANLIB              = _translate_bin_path(package:build_getenv("ranlib"))
     envs.CMAKE_RC_COMPILER         = _translate_bin_path(package:build_getenv("mrc"))
     envs.CMAKE_C_FLAGS             = _get_cflags(package, opt)
@@ -461,29 +495,26 @@ function _get_configs_for_mingw(package, configs, opt)
     -- Avoid cmake to add the flags -search_paths_first and -headerpad_max_install_names on macOS
     envs.HAVE_FLAG_SEARCH_PATHS_FIRST = "0"
     -- CMAKE_MAKE_PROGRAM may be required for some CMakeLists.txt (libcurl)
-    if is_subhost("windows") then
+    if is_subhost("windows") and opt.cmake_generator ~= "Ninja" then
         local mingw = assert(package:build_getenv("mingw") or package:build_getenv("sdk"), "mingw not found!")
         envs.CMAKE_MAKE_PROGRAM = path.join(mingw, "bin", "mingw32-make.exe")
     end
-
     if opt.cmake_generator == "Ninja" then
         envs.CMAKE_MAKE_PROGRAM = "ninja"
     end
-
-    for k, v in pairs(envs) do
-        table.insert(configs, "-D" .. k .. "=" .. v)
-    end
+    _insert_configs_from_envs(configs, envs, opt)
 end
 
 -- get configs for wasm
 function _get_configs_for_wasm(package, configs, opt)
+    opt = opt or {}
     local emsdk = find_emsdk()
     assert(emsdk and emsdk.emscripten, "emscripten not found!")
     local emscripten_cmakefile = find_file("Emscripten.cmake", path.join(emsdk.emscripten, "cmake/Modules/Platform"))
     assert(emscripten_cmakefile, "Emscripten.cmake not found!")
     table.insert(configs, "-DCMAKE_TOOLCHAIN_FILE=" .. emscripten_cmakefile)
-    if is_subhost("windows") then
-        local mingw = assert(package:build_getenv("mingw") or package:build_getenv("sdk"), "mingw not found!")
+    if is_subhost("windows") and opt.cmake_generator ~= "Ninja" then
+        local mingw = assert(package:build_getenv("mingw") or package:build_getenv("sdk"), "mingw32-make not found!")
         table.insert(configs, "-DCMAKE_MAKE_PROGRAM=" .. _translate_paths(path.join(mingw, "bin", "mingw32-make.exe")))
     end
     _get_configs_for_generic(package, configs, opt)
@@ -505,19 +536,27 @@ function _get_configs_for_cross(package, configs, opt)
         local dir = path.directory(cxx)
         local name = path.filename(cxx)
         name = name:gsub("clang$", "clang++")
-        name = name:gsub("clang%-", "clang++-")
+        name = name:gsub("clang%-", "clang++-") -- clang-xx
+        name = name:gsub("clang%.", "clang++.") -- clang.exe
         name = name:gsub("gcc$", "g++")
         name = name:gsub("gcc%-", "g++-")
-        envs.CMAKE_CXX_COMPILER = _translate_bin_path(dir and path.join(dir, name) or name)
+        name = name:gsub("gcc%.", "g++.")
+        if dir and dir ~= "." then
+            cxx = path.join(dir, name)
+        else
+            cxx = name
+        end
+        envs.CMAKE_CXX_COMPILER = _translate_bin_path(cxx)
     end
     -- @note The link command line is set in Modules/CMake{C,CXX,Fortran}Information.cmake and defaults to using the compiler, not CMAKE_LINKER,
-    -- so we need set CMAKE_CXX_LINK_EXECUTABLE to use CMAKE_LINKER as linker.
+    -- so we need to set CMAKE_CXX_LINK_EXECUTABLE to use CMAKE_LINKER as linker.
     --
     -- https://github.com/xmake-io/xmake-repo/pull/1039
     -- https://stackoverflow.com/questions/1867745/cmake-use-a-custom-linker/25274328#25274328
-    envs.CMAKE_LINKER              = _translate_bin_path(package:build_getenv("ld"))
+    -- https://github.com/xmake-io/xmake-repo/pull/2134#issuecomment-1573195810
+    local ld = _translate_bin_path(package:build_getenv("ld"))
     if package:has_tool("ld", "gxx", "clangxx") then
-        envs.CMAKE_CXX_LINK_EXECUTABLE = "<CMAKE_LINKER> <FLAGS> <CMAKE_CXX_LINK_FLAGS> <LINK_FLAGS> <OBJECTS> -o <TARGET> <LINK_LIBRARIES>"
+        envs.CMAKE_CXX_LINK_EXECUTABLE = ld .. " <FLAGS> <CMAKE_CXX_LINK_FLAGS> <LINK_FLAGS> <OBJECTS> -o <TARGET> <LINK_LIBRARIES>"
     end
     envs.CMAKE_RANLIB              = _translate_bin_path(package:build_getenv("ranlib"))
     envs.CMAKE_C_FLAGS             = _get_cflags(package, opt)
@@ -526,10 +565,14 @@ function _get_configs_for_cross(package, configs, opt)
     envs.CMAKE_STATIC_LINKER_FLAGS = table.concat(table.wrap(package:build_getenv("arflags")), ' ')
     envs.CMAKE_EXE_LINKER_FLAGS    = _get_ldflags(package, opt)
     envs.CMAKE_SHARED_LINKER_FLAGS = _get_shflags(package, opt)
-    -- we need not set it as cross compilation if we just pass toolchain
+    -- we don't need to set it as cross compilation if we just pass toolchain
     -- https://github.com/xmake-io/xmake/issues/2170
     if not package:is_plat(os.subhost()) then
-        envs.CMAKE_SYSTEM_NAME     = "Linux"
+        local system_name = package:targetos() or "Linux"
+        if system_name == "linux" then
+            system_name = "Linux"
+        end
+        envs.CMAKE_SYSTEM_NAME = system_name
     else
         if package:config("pic") ~= false then
             table.insert(configs, "-DCMAKE_POSITION_INDEPENDENT_CODE=ON")
@@ -537,20 +580,18 @@ function _get_configs_for_cross(package, configs, opt)
     end
     -- avoid find and add system include/library path
     -- @see https://github.com/xmake-io/xmake/issues/2037
-    envs.CMAKE_FIND_ROOT_PATH      = sdkdir
+    envs.CMAKE_FIND_ROOT_PATH              = sdkdir
     envs.CMAKE_FIND_ROOT_PATH_MODE_LIBRARY = "BOTH"
     envs.CMAKE_FIND_ROOT_PATH_MODE_INCLUDE = "BOTH"
     envs.CMAKE_FIND_ROOT_PATH_MODE_PROGRAM = "NEVER"
     -- avoid add -isysroot on macOS
     envs.CMAKE_OSX_SYSROOT = ""
-    -- Avoid cmake to add the flags -search_paths_first and -headerpad_max_install_names on macOS
+    -- avoid cmake to add the flags -search_paths_first and -headerpad_max_install_names on macOS
     envs.HAVE_FLAG_SEARCH_PATHS_FIRST = "0"
-    -- Avoids finding host include/library path
+    -- avoids finding host include/library path
     envs.CMAKE_FIND_USE_CMAKE_SYSTEM_PATH = "0"
     envs.CMAKE_FIND_USE_INSTALL_PREFIX = "0"
-    for k, v in pairs(envs) do
-        table.insert(configs, "-D" .. k .. "=" .. v)
-    end
+    _insert_configs_from_envs(configs, envs, opt)
 end
 
 -- get configs for host toolchain
@@ -562,6 +603,7 @@ function _get_configs_for_host_toolchain(package, configs, opt)
     envs.CMAKE_C_COMPILER          = _translate_bin_path(package:build_getenv("cc"))
     envs.CMAKE_CXX_COMPILER        = _translate_bin_path(package:build_getenv("cxx"))
     envs.CMAKE_ASM_COMPILER        = _translate_bin_path(package:build_getenv("as"))
+    envs.CMAKE_RC_COMPILER         = _translate_bin_path(package:build_getenv("mrc"))
     envs.CMAKE_AR                  = _translate_bin_path(package:build_getenv("ar"))
     -- https://github.com/xmake-io/xmake-repo/pull/1096
     local cxx = envs.CMAKE_CXX_COMPILER
@@ -570,18 +612,26 @@ function _get_configs_for_host_toolchain(package, configs, opt)
         local name = path.filename(cxx)
         name = name:gsub("clang$", "clang++")
         name = name:gsub("clang%-", "clang++-")
+        name = name:gsub("clang%.", "clang++.")
         name = name:gsub("gcc$", "g++")
         name = name:gsub("gcc%-", "g++-")
-        envs.CMAKE_CXX_COMPILER = _translate_bin_path(dir and path.join(dir, name) or name)
+        name = name:gsub("gcc%.", "g++.")
+        if dir and dir ~= "." then
+            cxx = path.join(dir, name)
+        else
+            cxx = name
+        end
+        envs.CMAKE_CXX_COMPILER = _translate_bin_path(cxx)
     end
     -- @note The link command line is set in Modules/CMake{C,CXX,Fortran}Information.cmake and defaults to using the compiler, not CMAKE_LINKER,
     -- so we need set CMAKE_CXX_LINK_EXECUTABLE to use CMAKE_LINKER as linker.
     --
     -- https://github.com/xmake-io/xmake-repo/pull/1039
     -- https://stackoverflow.com/questions/1867745/cmake-use-a-custom-linker/25274328#25274328
-    envs.CMAKE_LINKER              = _translate_bin_path(package:build_getenv("ld"))
+    -- https://github.com/xmake-io/xmake-repo/pull/2134#issuecomment-1573195810
+    local ld = _translate_bin_path(package:build_getenv("ld"))
     if package:has_tool("ld", "gxx", "clangxx") then
-        envs.CMAKE_CXX_LINK_EXECUTABLE = "<CMAKE_LINKER> <FLAGS> <CMAKE_CXX_LINK_FLAGS> <LINK_FLAGS> <OBJECTS> -o <TARGET> <LINK_LIBRARIES>"
+        envs.CMAKE_CXX_LINK_EXECUTABLE = ld .. " <FLAGS> <CMAKE_CXX_LINK_FLAGS> <LINK_FLAGS> <OBJECTS> -o <TARGET> <LINK_LIBRARIES>"
     end
     envs.CMAKE_RANLIB              = _translate_bin_path(package:build_getenv("ranlib"))
     envs.CMAKE_C_FLAGS             = _get_cflags(package, opt)
@@ -590,7 +640,7 @@ function _get_configs_for_host_toolchain(package, configs, opt)
     envs.CMAKE_STATIC_LINKER_FLAGS = table.concat(table.wrap(package:build_getenv("arflags")), ' ')
     envs.CMAKE_EXE_LINKER_FLAGS    = _get_ldflags(package, opt)
     envs.CMAKE_SHARED_LINKER_FLAGS = _get_shflags(package, opt)
-    -- we need not set it as cross compilation if we just pass toolchain
+    -- we don't need to set it as cross compilation if we just pass toolchain
     -- https://github.com/xmake-io/xmake/issues/2170
     if not package:is_plat(os.subhost()) then
         envs.CMAKE_SYSTEM_NAME     = "Linux"
@@ -599,9 +649,7 @@ function _get_configs_for_host_toolchain(package, configs, opt)
             table.insert(configs, "-DCMAKE_POSITION_INDEPENDENT_CODE=ON")
         end
     end
-    for k, v in pairs(envs) do
-        table.insert(configs, "-D" .. k .. "=" .. v)
-    end
+    _insert_configs_from_envs(configs, envs, opt)
 end
 
 -- get cmake generator for msvc
@@ -652,8 +700,15 @@ function _get_configs_for_generator(package, configs, opt)
         table.insert(configs, "-G")
         table.insert(configs, _get_cmake_generator_for_msvc(package))
     elseif package:is_plat("wasm") and is_subhost("windows") then
+        -- we attempt to use ninja if it exist
+        -- @see https://github.com/xmake-io/xmake/issues/3771
         table.insert(configs, "-G")
-        table.insert(configs, "MinGW Makefiles")
+        if find_tool("ninja") then
+            table.insert(configs, "Ninja")
+            opt.cmake_generator = "Ninja"
+        else
+            table.insert(configs, "MinGW Makefiles")
+        end
     else
         table.insert(configs, "-G")
         table.insert(configs, "Unix Makefiles")
@@ -683,13 +738,13 @@ function _get_configs(package, configs, opt)
         _get_configs_for_android(package, configs, opt)
     elseif package:is_plat("iphoneos", "watchos") or
         -- for cross-compilation on macOS, @see https://github.com/xmake-io/xmake/issues/2804
-        (package:is_plat("macosx") and not package:is_arch(os.subarch())) then
+        (package:is_plat("macosx") and (get_config("appledev") or not package:is_arch(os.subarch()))) then
         _get_configs_for_appleos(package, configs, opt)
     elseif package:is_plat("mingw") then
         _get_configs_for_mingw(package, configs, opt)
     elseif package:is_plat("wasm") then
         _get_configs_for_wasm(package, configs, opt)
-    elseif _is_cross_compilation(package) then
+    elseif package:is_cross() then
         _get_configs_for_cross(package, configs, opt)
     elseif package:config("toolchains") then
         -- we still need find system libraries,
@@ -708,7 +763,7 @@ end
 -- get build environments
 function buildenvs(package, opt)
 
-    -- we need bind msvc environments manually
+    -- we need to bind msvc environments manually
     -- @see https://github.com/xmake-io/xmake/issues/1057
     opt = opt or {}
     local envs = {}
@@ -716,7 +771,7 @@ function buildenvs(package, opt)
         envs = _get_msvc_runenvs(package)
     end
 
-    -- we need pass pkgconf for windows/mingw without msys2/cygwin
+    -- we need to pass pkgconf for windows/mingw without msys2/cygwin
     if package:is_plat("windows", "mingw") and is_subhost("windows") then
         local pkgconf = find_tool("pkgconf")
         if pkgconf then
@@ -725,10 +780,13 @@ function buildenvs(package, opt)
     end
 
     -- add environments for cmake/find_packages
+    -- and we need also find them from private libraries,
+    -- @see https://github.com/xmake-io/xmake-repo/pull/2553
     local CMAKE_LIBRARY_PATH = {}
     local CMAKE_INCLUDE_PATH = {}
     local CMAKE_PREFIX_PATH  = {}
-    for _, dep in ipairs(package:librarydeps()) do
+    local PKG_CONFIG_PATH = {}
+    for _, dep in ipairs(package:librarydeps({private = true})) do
         if dep:is_system() then
             local fetchinfo = dep:fetch()
             if fetchinfo then
@@ -738,11 +796,20 @@ function buildenvs(package, opt)
             end
         else
             table.join2(CMAKE_PREFIX_PATH, dep:installdir())
+            local pkgconfig = path.join(dep:installdir(), "lib", "pkgconfig")
+            if os.isdir(pkgconfig) then
+                table.insert(PKG_CONFIG_PATH, pkgconfig)
+            end
+            pkgconfig = path.join(dep:installdir(), "share", "pkgconfig")
+            if os.isdir(pkgconfig) then
+                table.insert(PKG_CONFIG_PATH, pkgconfig)
+            end
         end
     end
     envs.CMAKE_LIBRARY_PATH = path.joinenv(CMAKE_LIBRARY_PATH)
     envs.CMAKE_INCLUDE_PATH = path.joinenv(CMAKE_INCLUDE_PATH)
     envs.CMAKE_PREFIX_PATH  = path.joinenv(CMAKE_PREFIX_PATH)
+    envs.PKG_CONFIG_PATH    = path.joinenv(PKG_CONFIG_PATH)
     return envs
 end
 
@@ -961,6 +1028,9 @@ function install(package, configs, opt)
 
     -- init options
     opt = opt or {}
+    if (not opt.cmake_generator) and project.policy("package.cmake_generator.ninja") then
+        opt.cmake_generator = "Ninja"
+    end
 
     -- enter build directory
     local buildir = opt.buildir or package:buildir()
@@ -1015,3 +1085,4 @@ function install(package, configs, opt)
     end
     os.cd(oldir)
 end
+
